<?php

namespace Lean\Api\Controller\Debug;

use Lean\Api\{Controller, Entity, Form, Lib};
use Notes\Route\Attribute\Method\Route;
use Notes\Route\Attribute\Object\Route as RouteObj;
use Notes\Security\Attribute\Security;
use Notes\Security\Attribute\Taxus;
use Psr\Http\Message\{ResponseInterface, ServerRequestInterface};

#[Security(locked: false)]
## [Taxus(Entity\UserTypeEnum::Developer)]
#[RouteObj(base: "/lean/debug/errors")]
class Errors {

    use \Lean\Api\Lib\ControllerTrait;

    #[Route("/documentation", name: "lean.api:errors-doc", method: "GET", description: "Documentation section")]
    public function documentation(ServerRequestInterface $request, array $arguments) : ResponseInterface
    {
        return $this->renderMarkdown(getenv("LEAN_API_PROJECT_PATH") . "/meta/docs/debug/errors.md", [ Entity\Error::class, ], [ Form\Debug\ErrorContext::class ]);
    }

    #[Route("/", name: "lean.api:errors-list", method: [ "GET" ], description: "List every errors received")]
    public function list(ServerRequestInterface $request, array $arguments): ResponseInterface
    {
        $search = Entity\Error::searchRequest()->fromRequest($request);
        $collection = Entity\Error::repository()->filterServerRequest($search)->loadAll()->iterate(function($e) {
            unset($e->trace);
        });

        return $this->output($collection, $search->count);
    }

    #[Route("/{id:\d+}", name: "lean.api:errors-single", method: [ "GET" ], description: "Get a single error from it's ID")]
    public function single(ServerRequestInterface $request, array $arguments): ResponseInterface
    {
        $type = Entity\Error::class;
        $request = $this->searchEntityFromRequest($request, $type);
        $result = $request->getAttribute('lean.searchRequest');

        return $this->output($result[$type]->entity, $result[$type]->search->count);
    }

}
