<?php

namespace Lean\Api\Middleware;

use Laminas\Diactoros\Response\JsonResponse;
use Lean\Factory\HttpFactory;
use Picea\Ui\Method\FormHandler;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Lean\Api\{Factory\DebugFormFactoryInterface, Form, Entity};

class ApiRenderer implements MiddlewareInterface {

    public function __construct(
        protected DebugFormFactoryInterface $debugFormFactory,
    ) {
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
    {
        try {
            $response = $handler->handle($request);
        }
        catch(\Throwable $ex) {
            # $errorId = $this->saveError($request, $ex)->id;

            if ($this->awaitingJson($request)) {
                return HttpFactory::createJsonResponse([
                    'status' => 'failed',
                    'ts' => time(),
                    'data' => [
                       # 'error_id' => $errorId,
                        'message' => $ex->getMessage(),
                        'file' => $ex->getFile(),
                        'line' => $ex->getLine(),
                        'code' => $ex->getCode(),
                        'backtrace' => $ex->getTrace(),
                    ]
                ], 500);
            }

            throw $ex;
        }

        if ($response instanceof JsonResponse) {
            $payload = $response->getPayload();

            # For now, we match only response having a 'data' field
            if (isset($payload['data'])) {
                return HttpFactory::createJsonResponse([
                    'status' => 'success',
                    'ts' => time(),
                ] + $payload, 200);
            }
        }

        return $response;
    }

    protected function saveError(ServerRequestInterface $request, \Throwable $exception) : Entity\Error
    {
        $request = $request->withMethod('PUT')->withParsedBody([]);

        $context = $this->debugFormFactory->errorSaveContext($request);

        $context->sets([
            'message' => $exception->getMessage(),
            'code' => $exception->getCode(),
            'file' => $exception->getFile(),
            'line' => $exception->getLine(),
            'trace' => $exception->getTrace(),
        ]);

        $form = $this->debugFormFactory->errorSave($entity = new Entity\Error());

        new FormHandler($request, $form, $context);

        return $entity;
    }

    protected function awaitingJson(ServerRequestInterface $request) : bool
    {
        return str_contains(strtolower($request->getHeaderLine('content-type')), 'json');
    }
}
